/* *******************************************************************
   Webserver

   How it all works together:

   page   0.htm         includes the f.css in the header
                        a configuration page to set timezone and ntp server
          404           a response when the requested resource is not available
          204           a response when the browser will not get any content
   css    f.css         a css stylesheet for all on flash (program) memory
          zones.json    the JSON with time zones
          select.js     a javascript handling the dropdown/select of time zones

   ***************************************************************** */
#pragma once
#include "ntp.h"              // needs the update of the NTP configuration
#include "external.h"         // copy paste all needed timezones

// a list of pages to have them in the navigation bar
const char htmlNavigation[] = ("<nav><p>"
                               "<a href='/'>Home</a>"
                               " | <a href='1.htm'>Config</a>"
                               "</p></nav>\n");

// root - 0.htm
void handlePage() {
  String message;
  String reserve(5000);                // maximum size of page

  message =  F("<!DOCTYPE html>\n"
               "<html lang=\"en\">\n"  // english or any other language code you using for your page content
               "<head>\n"
               "<title>" TXT_BOARDNAME "</title>\n"
               "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\">\n"
               "<meta name=\"viewport\" content=\"width=device-width\">\n"
               "<link rel='stylesheet' type='text/css' href='f.css'>\n"
               "</head>\n");
  message += F("<body>\n");
  message += F("<header>\n"
               "<h1>" TXT_BOARDNAME "</h1>\n");
  message += htmlNavigation;  // just some Fixt text with a navigation bar
  message += F("</header>\n"
               "<main>\n");

  // print whatever you need on the web page:
  time_t now;                       // this are the seconds since Epoch (1970) - UTC
  tm tm;                            // the structure tm holds time information in a more convenient way
  time(&now);                       // read the current time
  localtime_r(&now, &tm);           // update the structure tm with the current time
  char buffer[42] {0};              // a buffer large enough to hold your output
  strftime (buffer, sizeof(buffer), "%Y-%m-%d %H:%M:%S", &tm);  // for different formats see https://cplusplus.com/reference/ctime/strftime/
  message += F("<article class='narrow'>\n"
               "<h2>Date & Time</h2>\n"
               "<p>local: ");
  message += buffer;
  message += F("</p>\n"
               "<p>");
  message += tzlong;
  message += F("</p>\n"
               "<p>");
  message += tzshort;                          
  message += F("</p>\n");
  message += F("</article>\n");
  // end of content
  message += F("</main>\n");
  // a footer line
  message += F("<footer>\n"
               "<p>Running since:");
  message += millis() / 1000;
  message += F("<p>\n"
               "</footer>\n");
  message += F("</body>\n"
               "</html>\n");
  server.send(200, "text/html", message);
}

// NTP User Configuration - 1.htm
void handlePage1() {
  String message;
  String reserve(5000);                // maximum size of page

  bool updated = false;                // have we got new parameters
  if (server.hasArg("fallback") && server.arg("fallback") > "") {
    tzshort = server.arg("fallback");
    tzlong = "?"; 
    preferences.putString("tzlong", tzlong);
    preferences.putString("tzshort", tzshort);
    updated = true;
  } else if (server.hasArg("tzlong") && server.hasArg("tzshort") && server.arg("tzshort") != "?") {
    tzlong = server.arg("tzlong");
    tzshort = server.arg("tzshort");
    preferences.putString("tzlong", tzlong);
    preferences.putString("tzshort", tzshort);
    updated = true;
  } 
  if (server.hasArg("ntpserver")) {
    ntpServer = server.arg("ntpserver");
    preferences.putString("ntpServer", ntpServer);
    updated = true;
  }

  if (updated) {
    Serial.print("ntpServer="); Serial.println(ntpServer);
    Serial.print("tzshort="); Serial.println(tzshort);
    Serial.print("tzlong="); Serial.println(tzlong);
    ntpConfigTime(tzshort, ntpServer);
  }
  
  message =  F("<!DOCTYPE html>\n"
               "<html lang=\"en\">\n"  // english or any other language code you using for your page content
               "<head>\n"
               "<title>" TXT_BOARDNAME "</title>\n"
               "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\">\n"
               "<meta name=\"viewport\" content=\"width=device-width\">\n"
               "<link rel='stylesheet' type='text/css' href='f.css'>\n"
               "</head>\n");
  message += F("<body>\n");
  message += F("<header>\n"
               "<h1>" TXT_BOARDNAME "</h1>\n");
  message += htmlNavigation;  // just some fix text with a navigation bar
  message += F("</header>\n"
               "<main>\n");

  // print whatever you need on the web page:
  message += F("<article class='narrow'>\n"
               "<h2>NTP Configuration</h2>\n"
               "<form action='1.htm' method='POST'>\n"
               "  <select name='fallback' id='fw_combobox'>\n"   // we add some hardcoded timezones als fallback
               "    <option value=''>fallback - chose</option>\n"
               "    <option value='GMT0BST,M3.5.0/1,M10.5.0'>Europe/London</option>\n"
               "    <option value='CET-1CEST,M3.5.0,M10.5.0/3'>Europe/Berlin</option>\n"
               "    <option value='EET-2EEST,M3.5.0/3,M10.5.0/4'>Europe/Helsinki</option>\n"
               "    <option value='MSK-3'>Europe/Moscow</option>\n"
               "    <option value='<+05>-5'>Indian/Maldives</option>\n"
               "    <option value='<+06>-6'>Indian/Chagos</option>\n"
               "    <option value='<+0630>-6:30>'>Indian/Cocos</option>\n"
               "    <option value='WIB-7'>Asia/Jakarta</option>\n"
               "    <option value='HKT-8'>Asia/Hong_Kong</option>\n"
               "    <option value='<+08>-8'>Asia/Kuala_Lumpur</option>\n"
               "    <option value='<+10>-10'>Asia/Vladivostok</option>\n"
               "    <option value='EST5EDT,M3.2.0,M11.1.0'>America/New_York</option>\n"
               "    <option value='CST6CDT,M3.2.0,M11.1.0'>America/Chicago</option>\n"
               "    <option value='MST7MDT,M3.2.0,M11.1.0'>America/Denver</option>\n"
               "    <option value='PST8PDT,M3.2.0,M11.1.0'>America/Vancouver</option>\n"
               "    <option value='GMT0'>Etc/GMT</option>\n"
               "  </select><br>\n"
               "  <p> Area / Location </p>\n" 
               "  <p id='filter'></p>\n"  // here we will add two list boxes
               "  <p id='demo'></p>\n"    // this is only some place for debug messages
               "  <p>\n"
               "  <input type='text' id='tzlong' name='tzlong' class='ro' readonly style='display:none' value='");
  message += tzlong;
  message += F("'><br>\n"
               "  <input type='text' id='tzshort' name='tzshort' class='ro' readonly style='display:none' value='");
  message += tzshort;
  message += F("'><br>\n"
               "  <p>"
               "<label for='ntpserver'>NTP Server:</label><br>"
               "<input type='text' id='ntpserver' name='ntpserver' value='");
  message += ntpServer;             
  message += F("'></p>\n"
               "  <input type='submit' value='Submit'>\n"
               "  <script src='select.js'></script>\n"
               "</form>\n"
              );
  message += F("</article>\n");

  // end of content
  message += F("</main>\n");
  // a footer line
  message += F("<footer>\n"
               "<p>Running since:");
  message += millis() / 1000;
  message += F("<p>\n"
               "</footer>\n");
  message += F("</body>\n"
               "</html>\n");
  server.send(200, "text/html", message);
}

// output of stylesheet - f.css
void handleCss() {
  String message;
  String reserve(2000);                // maximum size of page
  message = F("*{font-family:sans-serif}\n"
              "body{margin:10px}\n"
              "h1, h2{color:white;background:#1279b1;text-align:center}\n"
              "h1{margin:1px;padding:5px;font-size:1.2em}\n"
              "h2{margin-top:1px;font-size:1.1em}\n"
              "main{text-align:center}\n"
              "nav{background-color:silver;margin:1px;padding:5px;font-size:0.8em}\n"
              "article{margin:0.2em;padding:0.1em;border-style:solid;border-width:thin;border-color:#C0C0C0;text-align:center;background-color:#E5E5E5}\n"
              "footer p{font-size:0.7em;color:dimgray;background:silver;text-align:center;margin-bottom:5px}\n"
              "input[type=button], input[type=submit], input[type=reset]{width:8em}\n"
              "label{text-align:right}\n"
              "input.ro{width:15em;background-color:#E5E5E5;border:0}\n" // read only styled like background
              ".narrow{margin:auto;width:20em}\n"                        // smaller display arrea
             );
  server.send(200, "text/css", message);
}

// output of timezones as JSON - zones.json
void handleZones() {
  server.send(200, "application/json", zones); // set MIME type https://www.ietf.org/rfc/rfc4627.txt
}

// JavaScript to handle the select/listboxes - select.js
void handleSelect() {
  String message = R"rawliteral(

// fetch zone.json, populate two select/listboxes
// inspired by https://github.com/madeindjs/select_option_filters
// full documentation https://werner.rothschopf.net/microcontroller/202402_arduino_esp_ntp_user_configuration.htm
// by noiasca 2024 
var myObj;  // global to be used in lookup functiona also

fetch("zones.json")
.then((response) => response.json())
.then((data) => {
  //console.log(myObj);
  myObj = data;
  add_first_word_filter('fw_combobox', function(){
    console.log('value applied');
  });
})
.catch((error) => console.error("Error loading JSON file", error));

// maps tzlong to tzshort: finds from "Africa/Abidjan" to "GMT0"
function tzlong2tzshort(tzlong) {
  for (const x in myObj) {
    if (tzlong == x) return myObj[x];
  }
  return "0";
}

// create a select tag with only first word group 
function add_first_word_filter(select_id, callback) {
  var select_tag = document.getElementById(select_id);
  var target = document.getElementById("filter")
  select_tag.style.display = 'none';                  // hide the fallback list box
  var firstSearch = document.createElement('select');
  var secondSearch = document.createElement('select');
  target.parentElement.insertBefore(firstSearch, target);
  target.parentElement.insertBefore(secondSearch, target);
  var words = [];
  let text = "";
  //document.getElementById("demo").innerHTML = text;
  for (const x in myObj) {
    // get the first word of the option text
    var option = x;                     // area/location Africa/Abidjan
    var optionTextWords = option.split("/");
    var optionTextFirstWord = optionTextWords[0];
    if( words.indexOf(optionTextFirstWord) == -1 ){
      words.push(optionTextFirstWord);
    }      
  }
  // insert words as option in firstSearch select
  for (var i = 0; i < words.length; i++) {
    var option = document.createElement('option');
    option.textContent = words[i];
    option.value = words[i];
    firstSearch.appendChild(option);
  }
  // listen to changes in first select
  firstSearch.addEventListener("change", function (e) {
    // erase old input
    secondSearch.innerHTML = "" ;
    // get the selected text
    var text = e.target.value;
    // loop on option, if option have the same text, we insert in in the second search
    for (const x in myObj){
      var option = x;
      var optionText = option;
      if( optionText.indexOf(text) != -1 ) {
        // delete first word of the option text
        var secondText = optionText.substr(optionText.indexOf("/") + 1);
        // create it & insert in on secondSearch
        var secondOption = document.createElement('option');
        secondOption.textContent = secondText;
        secondOption.value = secondText;
        secondSearch.appendChild(secondOption);
      }
    }
  });

  // listen second search to update main select value
  secondSearch.addEventListener("change", function (e) {
    // build the complete value
    var firstWord = firstSearch.options[firstSearch.selectedIndex].value;
    var secondPart = e.target.value;
    var result = firstWord + "/" + secondPart ;
    document.getElementById("tzlong").value = result;
    document.getElementById("tzshort").value = tzlong2tzshort(result);

    document.getElementById("tzlong").style.display='block';
    document.getElementById("tzshort").style.display='block';
    
    //select_tag.value = result ;  // brutto listbox is not complete - hence commented
    // document.getElementById("demo").innerHTML = result; // debug
    if(typeof callback != 'undefined'  ){
      callback()
    } 
    console.log(result);
  });
    
  // init the two new select
  var entry = document.getElementById("tzlong").value;    // remember current content from tzlong
  var words = entry.split("/");                           // split tzlong and set first and second
  firstSearch.value = words[0];                           // set firstSearch 
  var changeEvent = new Event('change');                  // Create a new 'change' event
  firstSearch.dispatchEvent(changeEvent);                 // needed to update secondSearch
  var secondText = entry.substr(entry.indexOf("/") + 1);  // second part may contain a / therefore don't use words[1]
  secondSearch.value = secondText;
  secondSearch.dispatchEvent(changeEvent);                // implizit update tzshort
  // final debug to see if we reached the end of the script
  //document.getElementById("demo").innerHTML='ok';  
}
  )rawliteral";

  server.send(200, "text/javascript", message);
}

// Output a "404 not found" page. It includes the parameters which comes handy for test purposes.
void handleNotFound() {
  Serial.println(F("D124 handleNotFound()"));
  String message;
  String reserve(5000);                // maximum size of page
  message += F("404 - File Not Found\n"
               "URI: ");
  message += server.uri();
  message += F("\nMethod: ");
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += F("\nArguments: ");
  message += server.args();
  message += F("\n");
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
}

// Output a "204 no content" page.
// Used for the favicon.ico request (as we don't have any favicon currently)
void handle204() {
  server.send(204);                    // this page doesn't send back content
}

// defines the resources and starts the webserver
void serverSetup() {
  server.on("/",            handlePage);       // home page
  server.on("/1.htm",       handlePage1);      // configuration of NTP
  server.on("/f.css",       handleCss);        // stylesheet
  server.on("/favicon.ico", handle204);        // not available, send 204
  server.on("/zones.json",  handleZones);    // the JSON with the time zone definitions
  server.on("/select.js",   handleSelect);     // JavaScript to generate two listboxes for the time zone
  server.onNotFound(handleNotFound);           // show a typical HTTP Error 404 page
  server.begin();                              // start the webserver
}
//
